import argparse
import coloredlogs
import logging
import os

from snowav.framework.framework import Snowav
from snowav.framework.process_day import process
from snowav.framework.point_values import run_point_values
from snowav.plotting.swe_volume import swe_volume
from snowav.plotting.image_change import image_change
from snowav.plotting.plotlims import plotlims as plotlims


def run():
    parser = argparse.ArgumentParser(description="Process AWSM model "
                                                 "results.")

    parser.add_argument('-f', '--snowav_config', dest='snowav_config',
                        type=str, help='Path to snowav configuration file.')

    parser.add_argument('--end_date', '--end-date', dest='end_date', type=str,
                        help='End date that will override what is given in the'
                             ' config file, intended for airflow application.')

    parser.add_argument('-t', '--topo_path', dest='topo_path', type=str,
                        help='Path to topo.nc file.')

    parser.add_argument('-d', '--nc_path', dest='nc_path', type=str,
                        help='Path to single snow.nc file for swe '
                             'volume calculation and figure without database '
                             'processing.')

    parser.add_argument('-v', '--value', dest='value', type=str,
                        default='swe_vol',
                        choices=['swe_z', 'swe_vol', 'swe_avail', 'swe_unavail'],
                        help='SWE value to calculate for single snow.nc file.')

    parser.add_argument('-s', dest='show', action='store_true',
                        help='Flag to display single snow.nc calculation.')

    parser.add_argument('-A', '--snow_a', dest='snow_a', type=str,
                        help='Path to "A" snow.nc file for simple difference, '
                             'must also include "-B", and must not use "-d".')

    parser.add_argument('-B', '--snow_b', dest='snow_b', type=str,
                        help='Path to "B" snow.nc file for simple difference, '
                             'must also include "-A", and must not use "-d".')

    parser.add_argument('-p', '--figs_path', dest='figs_path', type=str,
                        default=os.getcwd() + '/',
                        help='Path to save the figure, default is ./snowav/')

    parser.add_argument('-pc', '--point_values_config',
                        dest='point_values_config', type=str,
                        help='Path to point values config.ini file.')

    parser.add_argument('-pm', '--point_values_master',
                        dest='point_values_master', type=str,
                        help='Path to PointValuesCoreConfig.ini master config '
                             'file.')

    parser.add_argument('-pb', '--blank', dest='blank', action='store_true',
                        default=False, help='Make a blank example csv file '
                                            'for [basin] locations_csv')

    args = parser.parse_args()
    snowav_main(config_file=args.snowav_config, end_date=args.end_date,
                topo_path=args.topo_path, nc_path=args.nc_path,
                value=args.value, show=args.show, snow_a=args.snow_a,
                snow_b=args.snow_b, figs_path=args.figs_path,
                point_values_config=args.point_values_config,
                point_values_master=args.point_values_master,
                blank=args.blank)


def snowav_main(config_file=None, end_date=None, topo_path=None, nc_path=None,
                value=None, show=False, snow_a=None, snow_b=None,
                figs_path=None, point_values_config=None,
                point_values_master=None, blank=None):
    """ Command line function for snowav.

    Runs the standard snowav processing of AWSM files if config_file is
    present.

    """

    # standard snowav processing
    if config_file is not None:
        if not os.path.isfile(config_file):
            raise Exception('Invalid config file {} '.format(config_file))
        else:
            end_date = None

        Snowav(config_file=config_file, end_date=end_date)

    # point values processing
    point_values_args = [point_values_config, blank]

    if sum([x is not None for x in point_values_args]) > 0:
        run_point_values(point_values_config, point_values_master, blank)

    # single snow.nc file
    single_args = [topo_path, figs_path]
    if sum([x is not None for x in single_args]) > 1:
        diff = False

        if not os.path.isdir(figs_path):
            raise Exception('{} not a valid directory'.format(figs_path))

        if not os.path.isfile(topo_path):
            raise Exception('{} not a valid topo file'.format(topo_path))

        # single snow.nc
        if nc_path is not None:
            nc_path = [os.path.abspath(nc_path)]
            if not os.path.isfile(nc_path[0]):
                raise Exception('{} not a valid file name'.format(nc_path[0]))

        # difference between two snow.nc
        if snow_a is not None and snow_b is not None:
            diff = True
            nc_path = [os.path.abspath(snow_a)] + [os.path.abspath(snow_b)]

            for path in nc_path:
                if not os.path.isfile(path):
                    raise Exception('{} not a valid file name'.format(path))

        log = create_log()
        results = process(nc_path, os.path.abspath(topo_path), value, log)

        barcolors = ['xkcd:cobalt', 'xkcd:mustard green', 'xkcd:lichen',
                     'xkcd:pale green', 'xkcd:blue green', 'xkcd:bluish purple',
                     'xkcd:lightish purple', 'xkcd:deep magenta']

        fargs = {'show': show,
                 'print': False,
                 'masks': results['masks'],
                 'barcolors': barcolors,
                 'plotorder': results['plotorder'],
                 'lims': plotlims(results['plotorder']),
                 'vollbl': 'TAF',
                 'elevlbl': 'ft',
                 'depthlbl': 'in',
                 'edges': results['edges'],
                 'labels': results['labels'],
                 'percent_max': 99.5,
                 'percent_min': 0.05,
                 'figsize': (10, 5),
                 'dpi': 250,
                 'dplcs': 1,
                 'xlims': (0, len(results['edges'])),
                 'figs_path': os.path.abspath(figs_path),
                 'directory': 'cli'}

        # difference between two snow.nc files
        if diff:
            if len(nc_path[0]) > 20:
                title = nc_path[1][-20::] + ' - ' + nc_path[0][-20::]
            else:
                title = nc_path

            df = results['df'][nc_path[1]] - results['df'][nc_path[0]]
            fargs['df'] = df
            fargs['image'] = results['outputs']['swe_z'][1] - \
                             results['outputs']['swe_z'][0]
            fargs['title'] = title

            log.info(' Difference generated from:\n      {} '
                     'subtract\n      {}'.format(nc_path[1], nc_path[0]))
            log.info(' Saved figure in {}'.format(os.path.abspath(figs_path)))

            image_change(fargs, None)

        else:
            if len(nc_path[0]) > 35:
                title = '...' + nc_path[0][-20::]
            else:
                title = nc_path

            fargs['image'] = results['outputs']['swe_z'][0] / 25.4
            fargs['title'] = '{}, {}'.format(value, title)
            fargs['df'] = results['df'][os.path.abspath(nc_path[0])]

            swe_volume(fargs, None)
            log.info(' Saved figure in {}'.format(figs_path))


def create_log(log_level='INFO'):
    """ Create logger. """

    level_styles = {'info': {'color': 'white'},
                    'notice': {'color': 'magenta'},
                    'verbose': {'color': 'blue'},
                    'success': {'color': 'green', 'bold': True},
                    'spam': {'color': 'green', 'faint': True},
                    'critical': {'color': 'red', 'bold': True},
                    'error': {'color': 'red'},
                    'debug': {'color': 'green'},
                    'warning': {'color': 'yellow'}}

    field_styles = {'hostname': {'color': 'magenta'},
                    'programname': {'color': 'cyan'},
                    'name': {'color': 'white'},
                    'levelname': {'color': 'white', 'bold': True},
                    'asctime': {'color': 'green'}}

    log_level = log_level.upper()

    numeric_level = getattr(logging, log_level, None)
    if not isinstance(numeric_level, int):
        raise ValueError('Invalid log level: %s' % log_level)

    fmt = '%(levelname)s:%(name)s:%(message)s'
    logging.basicConfig(level=numeric_level)
    coloredlogs.install(level=numeric_level,
                        fmt=fmt,
                        level_styles=level_styles,
                        field_styles=field_styles)

    log = logging.getLogger(__name__)

    return log


def can_i_snowav(config_file):
    """ Function that wraps snowav for test case.

    Args
    ------
    config_file : str
        path to config file
    """

    success = True
    try:
        Snowav(config_file=config_file)
    except Exception as e:
        print(e)
        success = False

    return success


if __name__ == '__main__':
    run()




from snowav.utils.MidpointNormalize import MidpointNormalize
import numpy as np
from matplotlib import pyplot as plt
import matplotlib.colors as mcolors
from mpl_toolkits.axes_grid1 import make_axes_locatable
import seaborn as sns
import copy
import cmocean
import matplotlib.patches as mpatches
import pandas as pd
from datetime import timedelta
import netCDF4 as nc
from snowav.utils.wyhr import calculate_wyhr_from_date, calculate_date_from_wyhr
from snowav.database.database import collect, make_session
import snowav.framework.figures
from sys import exit

def flt_image_change(args, logger = None):
    '''
    Difference in SWE from one day prior to flight updates to the day of flight
    updates.

    The file specified in [plots] update_file is used to get dates of flight
    updates and the masks for which portions of the basin were updated. The
    actual SWE differences are between the snow.nc files on the dates of the
    flights and the day immediately prior. [plots] update_numbers is a 1-based
    list of subsetting flights that can be applied.

    Args
    ------
    args : dict
        dictionary with required inputs, see swi() figure for more information.

    logger : list
        snowav logger
    '''

    file = args['update_file']
    update_numbers = args['update_numbers']
    start_date = args['start_date']
    end_date = args['end_date']
    flight_outputs = args['flight_outputs']
    pre_flight_outputs = args['pre_flight_outputs']
    masks = args['masks']
    lims = args['lims']
    barcolors = args['barcolors']
    edges = args['edges']
    connector = args['connector']
    plotorder = args['plotorder']
    e = False
    flight_diff_fig_names = []
    flight_delta_vol_df = {}

    p = nc.Dataset(file, 'r')

    if update_numbers is None:
        times = p.variables['time'][:]
    else:
        if max(update_numbers) > len(p.variables['time']) - 1:
            raise Exception('Max flight update_numbers greater than available '
                            'flights')
        else:
            times = p.variables['time'][update_numbers]

    # remove flight indices that might be after the report period
    idx = []

    for i, n in enumerate(times):
        date = calculate_date_from_wyhr(int(n),args['wy'])
        if date > end_date:
            idx = np.append(idx,i)

    times = np.delete(times, idx)
    ix = np.argsort(times)
    times = times[ix]

    if update_numbers is not None:
        update_numbers = [int(x) for x in update_numbers]

    for i,time in enumerate(times):
        if update_numbers is not None:
            depth = p.variables['depth'][update_numbers[ix[i]],:,:]
        else:
            depth = p.variables['depth'][ix[i],:,:]

        delta_swe = flight_outputs['swe_z'][i][:] - pre_flight_outputs['swe_z'][i][:]
        delta_swe = delta_swe * args['depth_factor']

        # We will always make the difference between the previous day
        start_date = flight_outputs['dates'][i] - timedelta(hours = 24)
        end_date = flight_outputs['dates'][i]

        try:
            delta_swe_byelev = collect(connector, args['plotorder'], args['basins'],
                                       start_date, end_date,'swe_vol',
                                       args['run_name'], args['edges'],'difference')
        except:
            e = True

        try:
            end_swe_byelev = collect(connector, args['plotorder'], args['basins'],
                                     start_date, end_date, 'swe_vol',
                                     args['run_name'], args['edges'],'end')
        except:
            e = True

        if e:
            if logger is not None:
                logger.info(' Failed requesting database records ending on {} '
                            'for flight difference figure. This may mean that '
                            '[run] directory has not been processed with '
                            '[snowav] run_name: {} for the periods in '
                            '[plots] update_file. Try subsetting with [plots] '
                            'update_numbers or processing the full [run] '
                            'directory.'.format(end_date, args['run_name']))
                logger.info(' Flight figures being set to False...')

            # Returning empty lists will set flights to False
            return [], []

        # Make copy so that we can add nans for the plots
        delta_state = copy.deepcopy(delta_swe)
        vMin,vMax = np.nanpercentile(delta_state,[2,98])

        colorsbad = plt.cm.Set1_r(np.linspace(0., 1, 1))
        colors1 = cmocean.cm.matter_r(np.linspace(0., 1, 127))
        colors2 = plt.cm.Blues(np.linspace(0, 1, 128))
        colors = np.vstack((colorsbad,colors1, colors2))
        mymap = mcolors.LinearSegmentedColormap.from_list('my_colormap', colors)

        ixf = delta_state == 0
        delta_state[ixf] = -100000
        pmask = masks[plotorder[0]]['mask']
        ixo = pmask == 0
        delta_state[ixo] = np.nan
        cmap = copy.copy(mymap)
        cmap.set_bad('white',1.)

        sns.set_style('darkgrid')
        sns.set_context("notebook")

        plt.close(i)
        fig,(ax,ax1) = plt.subplots(num = i, figsize = args['figsize'],
                                    dpi = args['dpi'], nrows = 1, ncols = 2)

        norm = MidpointNormalize(midpoint=0, vmin=vMin-0.01, vmax=vMax+0.01)

        # this is primarily for awsm_test_cases, in which the flight update
        # nc file doesn't contain mask information. wy2019-forward flight nc
        # files *should* have a mask
        if hasattr(depth,'mask'):
            mask = np.ma.masked_array(depth.mask, ~depth.mask)
            if mask.shape != delta_state.shape:
                raise Exception('Dimensions {}: {} do not match snow.nc: {}'.format(
                                file, mask.shape, delta_state.shape))

            delta_state[mask] = np.nan
            h = ax.imshow(delta_state*mask, cmap = cmap, norm = norm)

        else:
            h = ax.imshow(delta_state, cmap = cmap, norm = norm)

        for name in masks:
            ax.contour(masks[name]['mask'],cmap = "Greys",linewidths = 1)

        h.axes.get_xaxis().set_ticks([])
        h.axes.get_yaxis().set_ticks([])
        divider = make_axes_locatable(ax)
        cax = divider.append_axes("right", size="5%", pad=0.2)
        cbar = plt.colorbar(h, cax = cax)
        cbar.set_label(r'$\Delta$ SWE [{}]'.format(args['depthlbl']))

        start_date = start_date + timedelta(hours=1)
        d = end_date + timedelta(hours=1)

        h.axes.set_title('Change in SWE from Snow Depth Update\n{}'
                         .format(start_date.date().strftime("%Y-%-m-%-d")))

        if args['dplcs'] == 0:
            tlbl = '{}: {} {}'.format(plotorder[0],
                                 str(int(delta_swe_byelev[plotorder[0]].sum())),
                                 args['vollbl'])
        else:
            tlbl = '{}: {} {}'.format(plotorder[0],
                                 str(np.round(delta_swe_byelev[plotorder[0]].sum(),
                                              args['dplcs'])),args['vollbl'])

        for iters,name in enumerate(lims.sumorder):
            if args['dplcs'] == 0:
                lbl = '{}: {} {}'.format(name,
                                    str(int(delta_swe_byelev[name].sum())),
                                    args['vollbl'])
            else:
                lbl = '{}: {} {}'.format(name,
                                    str(np.round(delta_swe_byelev[name].sum(),
                                    args['dplcs'])),args['vollbl'])

            if iters == 0:
                ax1.bar(range(0,len(args['edges'])),delta_swe_byelev[name],
                        color = barcolors[iters],
                        edgecolor = 'k',label = lbl)

            else:
                ax1.bar(range(0,len(args['edges'])),delta_swe_byelev[name],
                        color = barcolors[iters],
                        edgecolor = 'k',
                        label = lbl,
                        alpha = 0.5)

        end_swe_byelev = end_swe_byelev.fillna(0)
        datestr = flight_outputs['dates'][i].date().strftime("%Y%m%d")
        percent_delta_byelev = (delta_swe_byelev.sum(skipna=True)/end_swe_byelev.sum())*100
        delta_swe_byelev.to_csv('{}flight_{}_{}_taf.csv'.format(args['figs_path'],
                                                    args['directory'],datestr))
        percent_delta_byelev.to_csv('{}flight_{}_{}_percent.csv'.format(args['figs_path'],
                                                    args['directory'],datestr))

        flight_delta_vol_df[flight_outputs['dates'][i].date().strftime("%Y%m%d")] = delta_swe_byelev

        plt.tight_layout()
        xts = ax1.get_xticks()
        edges_lbl = []
        for x in xts[0:len(xts)-1]:
            edges_lbl.append(str(int(edges[int(x)])))

        ax1.set_xticklabels(str(x) for x in edges_lbl)
        for tick in ax1.get_xticklabels():
            tick.set_rotation(30)

        ylims = ax1.get_ylim()
        ymax = ylims[1] + abs(ylims[1] - ylims[0])
        ymin = ylims[0] - abs(ylims[1] - ylims[0])*0.1
        ax1.set_ylim((ymin, ymax))

        ax1.set_ylabel('{}'.format(args['vollbl']))
        ax1.set_xlabel('elevation [{}]'.format(args['elevlbl']))
        ax1.axes.set_title('Change in SWE')

        ax1.yaxis.set_label_position("right")
        ax1.tick_params(axis='x')
        ax1.tick_params(axis='y')
        ax1.yaxis.tick_right()

        if len(plotorder) > 1:
            ax1.legend(loc=(lims.legx,lims.legy))

        ax1.text(lims.btx,lims.bty,tlbl,horizontalalignment='center',
                 transform=ax1.transAxes,fontsize = 10)

        plt.tight_layout()
        fig.subplots_adjust(top=0.88)

        fig_name = '{}flight_{}_{}_diff.png'.format(args['figs_path'],args['directory'],datestr)
        flight_diff_fig_names += ['flight_{}_{}_diff.png'.format(args['directory'],datestr)]

        if logger is not None:
            logger.info(' saving {}'.format(fig_name))

        snowav.framework.figures.save_fig(fig, fig_name)

    p.close()

    return flight_diff_fig_names, flight_delta_vol_df



from datetime import datetime
import os

from snowav.database.database import collect


def write_properties(end_date, cnx, plotorder, basins, wy_start, run_name,
                     figs_path, values, vollbl='TAF', logger=None):
    """ Write daily total snowpack properties to csv.

    Args
    ------
    end_date {str}: end_date
    cnx {str}: database connector
    plotorder {list}: basins list
    basins {dict}: basins dict
    wy_start {str}: YYYY1001
    run_name {str}: snowav run_name
    figs_path {str}: path to save files
    values {list}: list of snowav values
    vollbl {str}: volume label
    depthlbl {str}: depth label
    logger {class}: logger
    """

    datestr = end_date.strftime("%Y%m%d")
    now_str = datetime.now().date().strftime("%Y-%m-%d")
    date_col = 'Date generated: {}'.format(now_str)
    unit = vollbl

    for value in values:
        out = collect(cnx, plotorder, basins, wy_start, end_date, value,
                      run_name, 'total', 'daily')

        out.index = out.index.date

        # setting index to date strips the index name
        out.index.name = 'date'

        if value.lower() == 'swe_vol':
            value_line = ('Snow Water Equivalent (SWE) volume in thousands ' +
                          'of acre-feet (TAF)')
        elif value.lower() == 'swi_vol':
            value_line = ('Surface Water Input (SWI) volume in thousands ' +
                          'of acre-feet (TAF)')
        else:
            if logger is not None:
                logger.warning(" Value types other than swe, swi, and "
                               "their derivates are not supported")
            return

        headers = ['USDA Agicultural Research Service Snowpack Summary Data',
                   value_line,
                   'Data provided are daily model results from the iSnobal model',
                   'First column is the date of model result',
                   'Second column is the total basin volume',
                   'Additional columns are the subbasins in the watershed',
                   date_col,
                   'Valid until next reports are generated',
                   'Contact: Scott Havens <scott.havens@usda.gov>'
                   '\n']

        filename = '{}_timeseries_{}_{}.csv'.format(value, datestr, unit.lower())
        path = os.path.join(os.path.abspath(figs_path), filename)

        if os.path.isfile(path):
            os.remove(path)

        with open(path, mode='w', encoding='utf-8') as f:
            f.write('\n'.join(headers))

        out.to_csv(path, encoding='utf-8', mode='a')

        if logger is not None:
            logger.info(' Saved: {}'.format(path))



import copy
from .gitinfo import __gitVersion__, __gitPath__
import os
import netCDF4 as nc
import numpy as np

from snowav.database.database import convert_watershed_names
from snowav import __version__
from tablizer.tablizer import summarize, store


def calculate(array, pixel, masks=None, method='sum', convert=None,
              units='TAF', decimals=3):
    '''
    Calculate values from iSnobal output snow.nc and em.nc files
    and convert to desired units:
        mean SWE depth
        total SWE volume
        total 'available' SWE volume (based on cold content)
        total 'unavailable' SWE volume (based on cold content)
        mean snow depth
        total SWI volume
        mean SWI depth
        mean evaporation
        total cold content
        mean density

    Applies all masks that are passed. Typically this is a basin mask, an
    elevation bin mask, and a snow mask if a 0 value has meaning for the
    calculated value (such as a mean density).

    This is referenced as an example in scripts.sample_process.py and README,
    so if changes are made those may require updating.

    Args
    ------
    array : np.array
        iSnobal model outputs from snow.nc or em.nc file
    pixel : int
        [m] model pixel size
    masks : list
        list of np.array or bool masks, optional
    method : str
        method of calcution, options are 'sum', 'mean'
    convert : str
        conversion property, options are 'depth', 'snow_depth', 'volume', None
    units : str
        conversion units, options are 'TAF', 'SI'
    decimals : int

    Returns
    ------
    value : np.array

    '''

    method_options = ['sum', 'mean']
    unit_options = ['TAF', 'SI']

    if method not in method_options:
        raise Exception('method options are {}'.format(method_options))

    if units not in unit_options:
        raise Exception('units options are {}'.format(unit_options))

    if units == 'TAF':

        # mm to inches
        if convert == 'depth':
            factor = 0.03937

        # m to inches
        elif convert == 'snow_depth':
            factor = 39.37

        # mm/pixel to TAF
        elif convert == 'volume':
            factor = (pixel ** 2) * 0.000000810713194 * 0.001

        else:
            factor = 1

    if units == 'SI':

        # mm to cm
        if convert == 'depth':
            factor = 0.1

        # m to cm
        elif convert == 'snow_depth':
            factor = 100

        # mm/pixel to MM^3
        elif convert == 'volume':
            factor = (pixel ** 2) * 0.000000810713194 * 1233.48 / 1e9

        else:
            factor = 1

    if convert is None:
        factor = 1

    if masks is not None:
        if type(masks) != list:
            masks = [masks]

        for i, mask in enumerate(masks):
            if mask.shape != array.shape:
                raise Exception('mask {}, {} and array {} do not '
                                'match'.format(i, mask.shape, array.shape))

            # use nan because output zero values have meaning
            mask = mask.astype('float')
            mask[mask < 1] = np.nan
            array = array * mask

    # make calculation and convert
    if method == 'sum':
        if np.sum(np.isnan(array)) == array.size:
            value = np.nan
        else:
            value = np.nansum(array) * factor

    if method == 'mean':
        value = np.nanmean(array) * factor

    if not np.isnan(value):
        value = value.round(decimals)

    return value


def snow_line(array, dem, masks=None, limit=0.01):
    """ Calculate mean snow line in image.

    Args
    ------
    array {arr}: snow.nc swe_z
    dem {arr}: dem
    masks {list}: list of masks to apply
    limit {float}: limit for swe_z line

    Returns
    ------
    value {int}: snow line
    """

    if masks is not None:
        if type(masks) != list:
            masks = [masks]

        for i, mask in enumerate(masks):
            if mask.shape != array.shape:
                raise Exception('mask {}, {} and array {} do not '
                                'match'.format(i, mask.shape, array.shape))

            # use nan because output zero values have meaning
            mask = mask.astype('float')
            mask[mask < 1] = np.nan
            array = array * mask

    ix = array > limit
    if np.sum(ix) > 0:
        value = int(np.nanpercentile(dem[ix].flatten(), [2, 90])[0])
    else:
        value = np.nan

    return value


def masks(dempath, convert, plotorder=None, plotlabels=None):
    '''
    Load dem, build snowav masks dictionary from topo.nc file. See
    CoreConfig.ini for more information on some config options and behavior.

    Currently 'Cherry Creek' is an exception.

    Args
    ------
    dempath : str
        Path to topo.nc file, which is intended to be built by basin_setup
        package.
    convert : bool
        Convert watershed name to existing snowav database
    plotorder : list
        Optional, default is all mask strings that appear in topo.nc file found
        in dempath.
    plotlabels : list
        Optional, default is mask_names.

    Returns
    ------
    out : dict

    '''

    out = {}
    masks = {}
    labels = {}
    logger = []

    if not os.path.isfile(dempath):
        raise Exception('dempath {} not a valid file'.format(dempath))

    ncf = nc.Dataset(dempath, 'r')
    dem = ncf.variables['dem'][:]
    veg_type = ncf.variables['veg_type'][:]

    mask_names = [ncf.variables['mask'].long_name]
    mask_names_total = ncf.variables['mask'].long_name

    for v in ncf.variables.items():

        # without the space before mask we get the basin total twice
        if ' mask' in v[0]:
            mask_names += [v[1].long_name]

    if plotorder is None:
        plotorder = mask_names
        plotorder = ['Cherry Creek' if x == 'Cherry' else x for x in plotorder]

    else:
        if len(plotorder) > len(mask_names):
            raise Exception(' Config option [snowav] masks: {} for {}, but '
                            'found {}'.format(plotorder, dempath, mask_names))

        for lbl in plotorder:
            if (lbl not in mask_names) and (lbl != 'Cherry Creek'):
                raise Exception(' Config option [snowav] masks: {} for {}, but '
                                'found {}'.format(plotorder, dempath, mask_names))

    if plotlabels is None:
        for name in plotorder:
            labels[name] = name

    else:
        if len(plotlabels) != len(plotorder):
            logger.append(' Config option [snowav] plotlabels {} not equal to '
                          '{}, setting plotlabels to '
                          'defaults'.format(plotlabels, plotorder))
            for name in plotorder:
                labels[name] = name

        elif plotlabels is None:
            for name in plotorder:
                labels[name] = name

        else:
            for i, name in enumerate(plotorder):
                logger.append(' Assigning plot label {} '.format(plotlabels[i]) +
                              'for topo.nc long_name field {}'.format(name))
                labels[name] = plotlabels[i]

    nrows = len(dem[:, 0])
    ncols = len(dem[0, :])

    for lbl in plotorder:

        if lbl == 'Cherry Creek':
            nclbl = 'Cherry'
        else:
            nclbl = lbl

        if lbl != mask_names_total:
            masks[lbl] = {'mask': ncf[nclbl + ' mask'][:], 'label': lbl}
        else:
            masks[lbl] = {'mask': ncf['mask'][:], 'label': nclbl}

    ncf.close()

    if convert:
        old_key = plotorder[0]
        new_key = convert_watershed_names(old_key)
        masks[new_key] = masks.pop(old_key)
        labels[new_key] = labels.pop(old_key)

    out['dem'] = dem
    out['veg_type'] = veg_type
    out['masks'] = masks
    out['nrows'] = nrows
    out['ncols'] = ncols
    out['plotorder'] = plotorder
    out['labels'] = labels
    out['logger'] = logger

    return out


def sum_precip(precip_path, percent_snow_path):
    """ Sum in place for precip and rain images.

    Args
    ------
    precip_path: path to precip.nc
    percent_snow_path: path to percent_snow.nc
    precip_total: array of total precip for the run
    rain_total: array of total rain for the run

    Returns
    ------
    precip: array, daily total precip
    rain: array, daily total rain
    """

    ppt = nc.Dataset(precip_path, 'r')
    percent_snow = nc.Dataset(percent_snow_path, 'r')

    # For the wy2019 daily runs, precip.nc always has an extra hour, but
    # in some WRF forecast runs there are fewer than 24...
    if len(ppt.variables['precip'][:]) > 24:
        nb = 24
    else:
        nb = len(ppt.variables['precip'][:])

    for nb in range(0, nb):
        pre = ppt['precip'][nb]
        ps = percent_snow['percent_snow'][nb]

        if nb == 0:
            rain = np.multiply(pre, (1 - ps))
            precip = copy.deepcopy(pre)
        else:
            rain = rain + np.multiply(pre, (1 - ps))
            precip = precip + copy.deepcopy(pre)

    # rain_total = rain_total + rain
    # precip_total = precip_total + precip

    ppt.close()
    percent_snow.close()

    return precip, rain


def input_summary(path, variable, methods, percentiles, database, location,
                  run_name, basin_id, run_id, masks):
    """ Summarize smrf outputs with tablizer.

    Args
    ------
    path {str}: nc path
    variable {str}: snowav variable
    methods {list}: list of summary methods
    percentiles {arr}: if using percentiles
    database {str}: connector string
    location {str}: database location
    run_name {str}: snowav run name
    basin_id {dict}: snowav basin id
    run_id {int}: snowav run_id
    masks {list}: masks to filter with
    """

    if not os.path.isfile(path):
        raise OSError('invalid file -> {}'.format(path))

    ncf = nc.Dataset(path, 'r')
    array = ncf.variables[variable][:]
    dates = ncf.variables['time'][:]
    date_units = ncf.variables['time'].units
    ncf.close()

    if len(array[:]) > 24:
        nb = 24
    else:
        nb = len(array[:])

    if variable in ['snow_density', 'precip_temp']:
        mask_zero_values = True
    else:
        mask_zero_values = False

    for nb in range(0, nb):
        date = nc.num2date(dates[nb], date_units)
        img = array[nb, :, :]
        results = summarize(img,
                            date,
                            methods,
                            percentiles=[percentiles[0], percentiles[1]],
                            decimals=3,
                            masks=masks,
                            mask_zero_values=mask_zero_values)
        store(results, variable, database, location, run_name, basin_id,
              run_id, date)


def getgitinfo():
    """gitignored file that contains specific SNOWAV version and path

    Input:
        - none
    Output:
        - path to base SNOWAV directory
    """
    # return git describe if in git tracked SMRF
    if len(__gitVersion__) > 1:
        return __gitVersion__

    # return overarching version if not in git tracked SMRF
    else:
        version = 'v' + __version__
        return version


def get_config_header():
    """
    Produces the string for the main header for the config file.
    """
    hdr = ("Configuration File for SNOWAV {0}\n").format(getgitinfo())

    return hdr


def get_snowav_path():
    """gitignored file that contains specific SNOWAV version and path

    Input:
        - none
    Output:
        - path to base SNOWAV directory
    """
    # find the absolute path and return
    snowav_path = os.path.abspath(__gitPath__)

    return snowav_path




from matplotlib import pyplot as plt
import matplotlib.dates as mdates
import seaborn as sns
import snowav.framework.figures
import numpy as np
from datetime import datetime

def diagnostics(args, logger):
    '''
    Simple model output diagnostics for the water year and report
    period, plots water year and report period mean precip [in], mean swe [in],
    snow line, and density.

    Args
    ----------
    args : dict
        dictionary with required inputs, see swi() figure for more information.

    logger : object

    '''
    if args['print']:
        print('diagnostics() figure args:\n')
        for name in args.keys():
            if name not in ['masks','image','swe']:
                print(name, ': ', args[name])

    basins = args['dbasins']
    depthlbl = args['depthlbl']
    elevlbl = args['elevlbl']
    start_date = args['start_date']
    barcolors = args['barcolors']
    barcolors.insert(0,'black')

    ix = args['snow_line'] == 0
    args['snow_line'][ix] = np.nan
    ix = args['density'] == 0
    args['density'][ix] = np.nan

    sns.set_style('darkgrid')
    sns.set_context('notebook')
    dfmt = mdates.DateFormatter('%b-%d')

    plt.close(0)
    f, a = plt.subplots(num = 0, figsize = (8,9), facecolor = 'white',
                        dpi = args['dpi'], nrows = 5, ncols = 2,
                        linewidth = 0.75)

    a = a.flatten()

    for iters, name in enumerate(basins):
        clr = barcolors[iters]

        # wy precip
        a[0].plot(args['precip'][name], label = name, color = clr, linewidth = 1)
        a[0].set_ylabel('precip [{}]'.format(depthlbl))

        # report period precip
        a[1].plot(args['precip_per'][name], label = name, color = clr, linewidth = 1)
        a[1].set_ylabel('precip [{}]'.format(depthlbl))
        m = args['precip_per'].max().max()
        if m < 0.25:
            m = 0.25
        a[1].set_ylim((-0.01, m + m*0.1))

        # wy SWE depth
        a[2].plot(args['swe'][name], label = name, color = clr, linewidth = 1)
        a[2].set_ylabel('swe [{}]'.format(depthlbl))

        # report period SWE depth
        a[3].plot(args['swe_per'][name], label = name, color = clr, linewidth = 1)
        a[3].set_ylabel('$\Delta$ swe [{}]'.format(depthlbl))

        # wy density
        a[4].plot(args['density'][name], label = name, color = clr, linewidth = 1)
        a[4].set_ylabel(r'density [$kg/m^3$]')
        a[4].set_ylim((100,600))

        # report period density
        a[5].plot(args['density_per'][name], label = name, color = clr, linewidth = 1)
        a[5].set_ylabel(r'$\Delta$ density [$kg/m^3$]')

        # wy snow line
        a[6].plot(args['snow_line'][name], label = name, color = clr, linewidth = 1)
        a[6].set_ylabel('snow line [{}]'.format(elevlbl))

        # report period snow line
        a[7].plot(args['snow_line_per'][name], label = name, color = clr, linewidth = 1)
        a[7].set_ylabel('$\Delta$ snow line [{}]'.format(elevlbl))

        # wy evap_z
        a[8].plot(args['evap_z'][name].cumsum(), label = name, color = clr, linewidth = 1)
        a[8].set_ylabel('evap_z [{}]'.format(depthlbl))

        # report period evap_z
        a[9].plot(args['evap_z_per'][name].cumsum(), label = name, color = clr, linewidth = 1)
        a[9].set_ylabel('$\Delta$ evap_z [{}]'.format(depthlbl))

    for n in range(0,len(a)):

        if args['flt_flag']:
            for i,d in enumerate(args['flight_dates']):
                if n == 0 and i == 0:
                    lb = 'flight'.format(args['wy'])
                else:
                    lb = '__nolabel__'

                a[n].axvline(x=d, linestyle=':', linewidth=0.75, color='r', label=lb)

        if n in [1,3,5,7,9]:
            a[n].yaxis.tick_right()
            a[n].yaxis.set_label_position('right')
            a[n].set_xlim((args['start_date'], args['end_date']))

        else:
            a[n].set_xlim((datetime(args['wy']-1, 10, 1), args['end_date']))

            if n == 0:
                lbl = 'report period'
            else:
                lbl = '__nolabel__'

            a[n].axvline(x=start_date, linestyle=':', linewidth=0.65, color='k',
                         label = lbl)

        if n < 8:
            a[n].set_xticklabels('')

        else:
            a[n].xaxis.set_major_formatter(dfmt)
            for tick in a[n].get_xticklabels():
                tick.set_rotation(45)

    a[0].set_title('Water Year')
    a[0].legend(loc='upper left', fontsize = 8)
    a[1].set_title('Report Period')
    plt.tight_layout()

    del barcolors[0]

    fig_name_short = 'diagnostics_'
    fig_name = '{}{}{}.png'.format(args['figs_path'],fig_name_short,args['directory'])
    if logger is not None:
        logger.info(' saving {}'.format(fig_name))
    snowav.framework.figures.save_fig(f, fig_name)

    return fig_name_short



import cmocean
import copy
from datetime import datetime
from matplotlib import pyplot as plt
import matplotlib.colors
from mpl_toolkits.axes_grid1 import make_axes_locatable
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
import numpy as np
import pandas as pd
import os
import seaborn as sns
import utm

import snowav.framework.figures


def point_values_csv(output, value, df, imgxy, filename, model_date, basin,
                     heading, logger):
    """Read in specified csv file of snow course validation locations, write
    model values at those points.

    Args
    ------
    output {arr}: model output array from date of interest
    value {str}: snowav database value
    df {DataFrame}: DataFrame from point_values_csv file
    imgxy {arr}: x, y coordinates for the image, from get_topo_stats.py
    filename {str}: file name
    model_date {str}: date that model values were pulled
    logger {class}: logger
    """

    if value == 'swe_z':
        header = 'SWE [in]'
        intro_col = 'Snow Water Equivalent (SWE) depth in inches'
    if value == 'depth':
        header = 'snow depth [in]'
        intro_col = 'Snow depth in inches'
    if value == 'density':
        header = 'density [kg/m^3]'
        intro_col = 'Snow density in kg/m^3'

    date_str = datetime.now().date().strftime("%Y-%m-%d")
    date_col = 'Date generated: {}'.format(date_str)
    basin_col = 'Model basin: {}'.format(basin)
    model_col = 'Model results from: {}'.format(model_date)
    med_col = '9 pixel median {}'.format(header)
    lat_col = 'latitude'
    lon_col = 'longitude'
    x_col = 'model x'
    y_col = 'model y'
    stn_col = 'name'
    out_cols = [stn_col, lat_col, lon_col, x_col, y_col, header, med_col]
    all_cols = [stn_col, x_col, y_col, lat_col, lon_col, header, med_col,
                heading]
    sic = 'The "{}" column refers the model value in the pixel that ' \
          'contains the "{}" "{}" point'.format(header, lat_col, lon_col)
    npmc = 'The "{}" column means the median value ' \
           'of the 9 closest pixels to the "{}" "{}" ' \
           'point'.format(med_col, lat_col, lon_col)

    file_header = ['USDA Agicultural Research Service Snowpack Summary Data',
                   intro_col,
                   'Data provided are model results from the iSnobal model',
                   basin_col,
                   model_col,
                   date_col,
                   'The "model x" column refers to the model column index',
                   'The "model y" column refers to the model row index',
                   sic,
                   npmc,
                   'Contact: Scott Havens <scott.havens@usda.gov>\n']

    pixel_swe = pd.DataFrame(columns=all_cols)

    # get closest model pixel
    for i in df.index:
        sta = df.loc[i, stn_col]
        lat = df.loc[i, lat_col]
        lon = df.loc[i, lon_col]

        ll = utm.from_latlon(lat, lon)
        xind = np.where(abs(imgxy[0] - ll[0]) == min(abs(imgxy[0] - ll[0])))[0]
        yind = np.where(abs(imgxy[1] - ll[1]) == min(abs(imgxy[1] - ll[1])))[0]

        row_lims = [0, len(output[:, 0]) - 1]
        col_lims = [0, len(output[0, :]) - 1]

        if xind in col_lims or yind in row_lims:
            logger.warn(' Skipping {} in point_values, latitude and/or '
                        'longitude may be outside of the model '
                        'domain'.format(sta))
        else:
            pixel_swe.loc[i, stn_col] = sta
            pixel_swe.loc[i, lat_col] = lat
            pixel_swe.loc[i, lon_col] = lon
            pixel_swe.loc[i, x_col] = int(xind)
            pixel_swe.loc[i, y_col] = int(yind)
            pixel_swe.loc[i, heading] = df.loc[i, heading]

            # Get the median of the nine closest pixels
            val = np.array([])
            for n in range(-1, 1):
                for m in range(-1, 1):
                    val = np.append(val, output[yind + n, xind + m])
                    if n == 0 and m == 0:
                        pixel_swe.loc[i, header] = \
                            np.round(float(output[yind + n, xind + m]),
                                     decimals=2)

            pixel_swe.loc[i, med_col] = \
                np.round(np.nanmedian(val), decimals=2)

    pixel_swe = pixel_swe.round(4)

    if os.path.isfile(filename):
        os.remove(filename)

    with open(filename, mode='w', encoding='utf-8') as f:
        f.write('\n'.join(file_header))

    pixel_swe.to_csv(filename, encoding='utf-8', mode='a', columns=out_cols,
                     index=False)
    logger.info(" Saved: {}".format(filename))

    return pixel_swe


def point_values_figures(output, value, df, dem, figs_path, veg_type,
                         model_date, course_date, settings, pixel, heading,
                         logger):
    """ Make point values figures.

    Args
    ------
    output {arr}: model output array from date of interest
    value {str}: snowav database value
    df {DataFrame}: DataFrame from point_values_csv function
    dem {arr}: dem
    figs_path {str}: base path for saving
    veg_type {arr}: veg type
    model_date {str}: date that model values were pulled
    course_date {str}: snow course data
    settings {list}: figure settings
    pixel {int}: pixel sixe
    heading {string}: df heading
    logger {class}: logger
    """

    if value == 'swe_z':
        header = 'SWE [in]'
    if value == 'depth':
        header = 'snow depth [in]'
    if value == 'density':
        header = 'density [kg/m^3]'

    x_col = 'model x'
    y_col = 'model y'
    stn_col = 'name'
    width = settings[0]
    height = settings[1]
    dpi = settings[2]
    nrows = settings[3]
    ncols = settings[4]
    font_small = settings[5]
    font_medium = settings[6]
    npix = settings[7]
    ss = settings[8]
    levels = settings[9]
    annot_x1 = settings[10]
    annot_y1 = settings[11]
    annot_x2 = settings[12]
    annot_y2 = settings[13]

    sns.set_style('white')
    plt.close(0)
    f, a = plt.subplots(num=0, figsize=(int(width), int(height)),
                        dpi=dpi, nrows=nrows, ncols=ncols)
    a = a.flatten()

    plt.close(1)
    f1, a1 = plt.subplots(num=1, figsize=(int(width), int(height)),
                          dpi=dpi, nrows=nrows, ncols=ncols)
    a1 = a1.flatten()

    plt.close(2)
    f2 = plt.figure(num=2, figsize=(width, height - 2), dpi=dpi)
    a2 = plt.gca()

    plt.close(3)
    f3 = plt.figure(num=3, figsize=(width, height - 2), dpi=400)
    a3 = plt.gca()

    lvls = list(np.arange(np.min(dem.flatten()), np.max(dem.flatten()), 500))

    snow_map = cmocean.cm.haline_r

    veg = []
    for n, idx in enumerate(df.index):

        ix = int(df.loc[idx][x_col])
        iy = int(df.loc[idx][y_col])

        veg_sub = veg_type[(iy - npix):(iy + npix + 1),
                  (ix - npix):(ix + npix + 1)].flatten()
        veg = np.append(veg_sub, veg)

        if n == 0:
            g_min = np.nanmin(output[(iy - npix):(iy + npix),
                              (ix - npix):(ix + npix)])
            g_max = np.nanmax(output[(iy - npix):(iy + npix),
                              (ix - npix):(ix + npix)])

        else:
            vmin = np.nanmin(output[(iy - npix):(iy + npix),
                             (ix - npix):(ix + npix)])
            vmax = np.nanmax(output[(iy - npix):(iy + npix),
                             (ix - npix):(ix + npix)])

            if vmin < g_min:
                g_min = vmin
            if vmax > g_max:
                g_max = vmax

    ymin = copy.deepcopy(g_min)
    veg = np.unique(veg)
    vl = list(veg)
    norm = matplotlib.colors.BoundaryNorm(vl, len(vl))

    h3 = a3.imshow(veg_type, cmap='Vega20', norm=norm, alpha=0.75)
    a3.contour(dem, colors='k', levels=lvls, linewidths=0.15)

    place = np.linspace(g_min, g_max, 256)

    for idx, v in enumerate(df.name.unique()):
        mflag = False

        if idx < len(df[stn_col].unique()) + 1:

            # this plots repeating values
            if len(df[df[stn_col] == v][stn_col].values) > 1:
                mflag = True
                vstr = mstr = ''

                for idx3 in range(0, len(df[df[stn_col] == v][stn_col].values)):
                    course = df.loc[idx + idx3][heading]
                    ix = int(df[df[stn_col] == v][x_col].values[idx3])
                    iy = int(df[df[stn_col] == v][y_col].values[idx3])
                    ixs = int(np.where(abs(place - course) ==
                                       min(abs(place - course)))[0])
                    color = snow_map(ixs)

                    if idx3 == 0:
                        vstr = '{}'.format(round(course, 1))
                        mstr = '{}'.format(output[iy, ix].round(1))
                    else:
                        vstr = vstr + ', {}'.format(round(course, 1))
                        mstr = mstr + ', {}'.format(output[iy, ix].round(1))

                    a[idx].scatter(x=ix, y=iy, marker='^', s=ss, c=color,
                                   edgecolors='k', linewidths=0.5)

                cstr = 'validation: {}\nmodel: {}'.format(vstr, mstr)
                a[idx].text(annot_x1,
                            annot_y1,
                            cstr,
                            horizontalalignment='left',
                            transform=a[idx].transAxes,
                            fontsize=font_small - 1,
                            color='w')

            ix = int(df[df[stn_col] == v][x_col].values[0])
            iy = int(df[df[stn_col] == v][y_col].values[0])
            swe = df[df[stn_col] == v][header].values[0]
            course = df[df[stn_col] == v][heading].values[0]

            s = dem[(iy - npix):(iy + npix + 1),
                    (ix - npix):(ix + npix + 1)]
            snow_sec = output[(iy - npix):(iy + npix + 1),
                              (ix - npix):(ix + npix + 1)]

            l1 = [int(x) * snow_sec.shape[0] +
                  npix - 1 for x in range(npix - 1, npix + 2)] + \
                 [int(x) * snow_sec.shape[0] +
                  npix for x in range(npix - 1, npix + 2)] + \
                 [int(x) * snow_sec.shape[0] +
                  npix + 1 for x in range(npix - 1, npix + 2)]

            if idx == 0:
                sl = r'$\pm${} pixels'.format(npix)
                bl1 = r'$\pm$1 pixel'.format(npix)
            else:
                sl = '__nolabel__'
                bl1 = '__nolabel__'

            b1 = a2.boxplot(snow_sec.flatten(), positions=[idx - 0.22],
                            widths=0.18, showfliers=False)
            b2 = a2.boxplot(snow_sec.flatten()[l1], positions=[idx + 0.22],
                            widths=0.18, showfliers=False)
            a2.plot(0, ymin, 'dimgray', label=sl)
            a2.plot(0, ymin, 'forestgreen', label=bl1)
            if idx == 0:
                astr = 'val-mod [in], %:\n{}\n' \
                       '{}%'.format(round((course - swe), 1),
                                    int((swe / course) * 100))
            else:
                astr = '{}\n{}%'.format(round((course - swe), 1),
                                        int((swe / course) * 100))

            a2.annotate(astr, (idx - 0.25, ymin - 2), fontsize=font_small,
                        color='k')

            for prop in ['boxes', 'whiskers', 'fliers', 'caps']:
                plt.setp(b1[prop], color='dimgray')
                plt.setp(b2[prop], color='forestgreen')

            lvls = list(np.arange(np.min(s.flatten()),
                                  np.max(s.flatten()), levels))
            ixs = int(np.where(abs(place - course) ==
                               min(abs(place - course)))[0])

            color = snow_map(ixs)
            a[idx].contour(dem, colors='k', levels=lvls, linewidths=0.25)
            h = a[idx].imshow(output, cmap=snow_map, clim=(g_min, g_max))

            if not mflag:
                a[idx].scatter(x=ix, y=iy, marker='^', s=ss, c=color,
                               edgecolors='k', linewidths=0.5)
                cstr = 'validation: {}\nmodel: ' \
                       '{}'.format(round(course, 1),
                                   output[iy, ix].round(1))
                a[idx].text(annot_x1, annot_y1, cstr,
                            horizontalalignment='left',
                            transform=a[idx].transAxes,
                            fontsize=font_small,
                            color='w')

            a[idx].get_xaxis().set_ticks([])
            a[idx].get_yaxis().set_ticks([])
            a[idx].set_ylim((iy + npix + 0.5), (iy - (npix + 0.5)))
            a[idx].set_xlim((ix - (npix + 0.5), ix + npix + 0.5))
            a[idx].set_title(v, fontsize=font_small)

            # legend of sorts
            veg_subp = veg_type[(iy - npix):(iy + npix + 1),
                       (ix - npix):(ix + npix + 1)]
            vegmap = plt.cm.get_cmap('Vega20')

            a1[idx].contour(dem, colors='k', levels=lvls, linewidths=0.25)
            a1[idx].imshow(veg_type, cmap='Vega20', norm=norm)
            a1[idx].scatter(x=ix, y=iy, marker='^', s=ss, c='k',
                            edgecolors='k', linewidths=0.5)
            a1[idx].get_xaxis().set_ticks([])
            a1[idx].get_yaxis().set_ticks([])

            a1[idx].set_ylim((iy + npix + 0.5), (iy - (npix + 0.5)))
            a1[idx].set_xlim((ix - (npix + 0.5), ix + npix + 0.5))
            a1[idx].set_title(v, fontsize=font_small)

            a3.scatter(x=ix, y=iy, marker='s', s=10, color="None",
                       edgecolors='k', linewidths=0.5)
            a3.annotate(v,
                        (ix + 5, iy - 5),
                        fontsize=font_small - 1,
                        color='k')

            for vs in np.unique(veg_subp):
                ixf = np.where(veg_subp == vs)
                fx = ixf[0][0]
                fy = ixf[1][0]
                nx = ix + fy - npix
                ny = iy + fx - npix
                a1[idx].scatter(x=nx, y=ny, marker='X', s=3, c='k',
                                edgecolors='k', linewidths=0.5)
                a1[idx].annotate(str(vs), (nx, ny), fontsize=font_small - 3,
                                 color='w')

    # tack on another subplot for legend and colorbar
    a[idx + 1].imshow(output, cmap=snow_map, alpha=0)
    a[idx + 1].set_title('', fontsize=font_small)
    a[idx + 1].get_xaxis().set_ticks([])
    a[idx + 1].get_yaxis().set_ticks([])
    a[idx + 1].axis('off')
    cbaxes = inset_axes(a[idx + 1], width="90%", height="10%", loc=8)
    cbar = f.colorbar(h,
                      cax=cbaxes,
                      ticks=[int(g_min + 1),
                             int(((g_max - g_min) / 2) + g_min),
                             int(g_max)],
                      orientation='horizontal')
    cbar.ax.tick_params(labelsize=font_medium)
    cbar.set_label(header, fontsize=font_medium)
    cbar.ax.xaxis.set_label_position('top')
    cstr = (r'Validation pixel $\pm${} pixels'
            '\nmodel: {}\nvalidation: {}\ncontour interval: {} '
            'ft\npixel width: {} m, {} '
            'ft'.format(npix, model_date, course_date,
                        levels, str(pixel), str(int(pixel * 3.28))))
    a[idx + 1].text(annot_x2, annot_y2, cstr, horizontalalignment='left',
                    transform=a[idx + 1].transAxes, fontsize=font_medium)

    a1[idx + 1].imshow(veg_type,
                       cmap=vegmap,
                       clim=(np.min(veg), np.max(veg)),
                       alpha=0)
    a1[idx + 1].set_title('', fontsize=font_small)
    a1[idx + 1].get_xaxis().set_ticks([])
    a1[idx + 1].get_yaxis().set_ticks([])
    a1[idx + 1].axis('off')
    cstr = ('model: {}\nvalidation: {}\ncontour interval: {} '
            'ft\npixel width: {} m, {} '
            'ft'.format(model_date, course_date,
                        levels, str(pixel), str(int(pixel * 3.28))))
    a1[idx + 1].text(annot_x2, annot_y2, cstr, horizontalalignment='left',
                     transform=a1[idx + 1].transAxes, fontsize=font_medium)

    for n in range(idx + 2, nrows * ncols):
        f.delaxes(a[n])
    for n in range(idx + 2, nrows * ncols):
        f1.delaxes(a1[n])

    a2.set_ylabel(header)
    a2.set_xticks(list(range(0, len(df[stn_col].unique()))))
    a2.set_xticklabels(df[stn_col].unique(), rotation=90,
                       fontsize=font_small - 1)
    a2.set_xlim((-1, len(df[stn_col].unique()) + 1))
    a2.legend(loc='upper left')
    a2.grid(linewidth=0.25)
    a2.set_title('Validation and Model Pixel Values, {}'.format(model_date))

    a3.get_xaxis().set_ticks([])
    a3.get_yaxis().set_ticks([])
    divider3 = make_axes_locatable(a3)
    cax3 = divider3.append_axes("right", size="4%", pad=0.2)
    cbar3 = plt.colorbar(h3, cax=cax3, ticks=list(veg))
    cbar3.ax.tick_params()
    cbar3.set_label('vegetation type classification')

    f.tight_layout()
    f1.tight_layout()
    f2.tight_layout()
    f3.tight_layout()

    fig_name_short = 'validation_list_{}'.format(value)
    fig_name = '{}{}.png'.format(figs_path, fig_name_short)
    logger.info(' Saved: {}'.format(fig_name))
    snowav.framework.figures.save_fig(f2, fig_name)

    # currently only passing the swe_z image
    if value == 'swe_z':
        fig_name_short = 'validation_map_{}'.format(value)
        fig_name = '{}{}.png'.format(figs_path, fig_name_short)
        logger.info(' Saved: {}'.format(fig_name))
        snowav.framework.figures.save_fig(f, fig_name)

    fig_name_short = 'validation_veg_map'
    fig_name = '{}{}.png'.format(figs_path, fig_name_short)
    if not os.path.isfile(os.path.abspath(fig_name)):
        logger.info(' Saved: {}'.format(fig_name))
        snowav.framework.figures.save_fig(f1, fig_name)

    fig_name_short = 'validation_locations'
    fig_name = '{}{}.png'.format(figs_path, fig_name_short)
    if not os.path.isfile(os.path.abspath(fig_name)):
        logger.info(' Saved: {}'.format(fig_name))
        snowav.framework.figures.save_fig(f3, fig_name)



from collections import OrderedDict
from copy import deepcopy
import numpy as np
import pandas as pd


class AwsmInputsOutputs(object):
    """ smrf input and iSnobal output variable definitions for processing,
    including bands, units, database table, and derivatives. """

    def __init__(self):

        self.vars = {'depth':
                         {'band': 0,
                          'nc_name': 'thickness',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'depth',
                          'description': 'snow depth',
                          'units': 'm',
                          'calculate': 'mean',
                          'table': 'Results',
                          'file': 'snow.nc'},
                     'density':
                         {'band': 1,
                          'nc_name': 'snow_density',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'density',
                          'description': 'density',
                          'units': 'kg/m^3',
                          'calculate': 'mean',
                          'table': 'Results',
                          'file': 'snow.nc'},
                     'swe_z':
                         {'band': 2,
                          'nc_name': 'specific_mass',
                          'derivatives': {'requires': ['coldcont'],
                                          'products': ['swe_vol', 'swe_avail',
                                                       'swe_unavail']},
                          'unit_type': 'depth',
                          'description': 'snow water equivalent depth',
                          'units': 'mm',
                          'calculate': 'mean',
                          'table': 'Results',
                          'file': 'snow.nc'},
                     'lwc': {'band': 3,
                             'nc_name': 'liquid_water',
                             'derivatives': {'requires': [], 'products': []},
                             'unit_type': 'depth',
                             'description': 'liquid water content',
                             'units': 'mm',
                             'calculate': 'mean',
                             'table': 'Results',
                             'file': 'snow.nc'},
                     'temp_surface':
                         {'band': 4,
                          'nc_name': 'temp_surf',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'temperature',
                          'description': 'surface layer temperature',
                          'units': 'C',
                          'calculate': 'mean',
                          'table': 'Results',
                          'file': 'snow.nc'},
                     'temp_lower':
                         {'band': 5,
                          'nc_name': 'temp_lower',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'temperature',
                          'description': 'lower layer temperature',
                          'units': 'C',
                          'calculate': 'mean',
                          'table': 'Results',
                          'file': 'snow.nc'},
                     'temp_bulk':
                         {'band': 6,
                          'nc_name': 'temp_snowcover',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'temperature',
                          'description': 'bulk snowpack temperature',
                          'units': 'C',
                          'calculate': 'mean',
                          'table': 'Results',
                          'file': 'snow.nc'},
                     'depth_lower_layer':
                         {'band': 7,
                          'nc_name': 'thickness_lower',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'depth',
                          'description': 'lower layer depth',
                          'units': 'm',
                          'calculate': 'mean',
                          'table': 'Results',
                          'file': 'snow.nc'},
                     'h20_sat':
                         {'band': 8,
                          'nc_name': 'water_saturation',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'percent',
                          'description': '% liquid water saturation',
                          'units': '%',
                          'calculate': 'mean',
                          'table': 'Results',
                          'file': 'snow.nc'},
                     'R_n':
                         {'band': 0,
                          'nc_name': 'net_rad',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'flux',
                          'description': 'net all-wave radiation',
                          'units': 'W/m^2',
                          'calculate': 'mean',
                          'table': 'Results',
                          'file': 'em.nc'},
                     'H':
                         {'band': 1,
                          'nc_name': 'sensible_heat',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'flux',
                          'description': 'sensible heat transfer',
                          'units': 'W/m^2',
                          'calculate': 'mean',
                          'table': 'Results',
                          'file': 'em.nc'},
                     'L_v_E':
                         {'band': 2,
                          'nc_name': 'latent_heat',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'flux',
                          'description': 'latent heat exchange',
                          'units': 'W/m^2',
                          'calculate': 'mean',
                          'table': 'Results',
                          'file': 'em.nc'},
                     'G':
                         {'band': 3,
                          'nc_name': 'snow_soil',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'flux',
                          'description': 'snow/soil heat exchange',
                          'units': 'W/m^2',
                          'calculate': 'mean',
                          'table': 'Results',
                          'file': 'em.nc'},
                     'M':
                         {'band': 4,
                          'nc_name': 'precip_advected',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'flux',
                          'description': 'advected heat from precip',
                          'units': 'W/m^2',
                          'calculate': 'mean',
                          'table': 'Results',
                          'file': 'em.nc'},
                     'delta_Q':
                         {'band': 5,
                          'nc_name': 'sum_EB',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'flux',
                          'description': 'sum of e.b. terms',
                          'units': 'W/m^2',
                          'calculate': 'mean',
                          'table': 'Results',
                          'file': 'em.nc'},
                     'evap_z':
                         {'band': 6,
                          'nc_name': 'evaporation',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'depth',
                          'description': 'evaporation',
                          'units': 'mm',
                          'calculate': 'mean',
                          'table': 'Results',
                          'file': 'em.nc'},
                     'melt':
                         {'band': 7,
                          'nc_name': 'snowmelt',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'depth',
                          'description': 'melt',
                          'units': 'mm',
                          'calculate': 'mean',
                          'table': 'Results',
                          'file': 'em.nc'},
                     'swi_z':
                         {'band': 8,
                          'nc_name': 'SWI',
                          'derivatives': {'requires': [],
                                          'products': ['swi_vol']},
                          'unit_type': 'depth',
                          'description': 'surface water input depth',
                          'units': 'mm',
                          'calculate': 'mean',
                          'table': 'Results',
                          'file': 'em.nc'},
                     'coldcont':
                         {'band': 9,
                          'nc_name': 'cold_content',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'flux',
                          'description': 'cold content',
                          'units': 'J/m^2',
                          'calculate': 'mean',
                          'table': 'Results',
                          'file': 'em.nc'},
                     'air_temp':
                         {'band': None,
                          'nc_name': 'air_temp',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'temperature',
                          'description': 'air temperature',
                          'units': 'C',
                          'calculate': 'mean',
                          'table': 'Inputs',
                          'file': 'air_temp.nc'},
                     'cloud_factor':
                         {'band': None,
                          'nc_name': 'cloud_factor',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'dimensionless',
                          'description': 'cloud factor',
                          'units': 'unitless',
                          'calculate': 'mean',
                          'table': 'Inputs',
                          'file': 'cloud_factor.nc'},
                     'net_solar':
                         {'band': None,
                          'nc_name': 'net_solar',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'flux',
                          'description': 'net solar',
                          'units': 'W/m^2',
                          'calculate': 'mean',
                          'table': 'Inputs',
                          'file': 'net_solar.nc'},
                     'storm_days':
                         {'band': None,
                          'nc_name': 'storm_days',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'time',
                          'description': 'days since last precip',
                          'units': 'day',
                          'calculate': 'mean',
                          'table': 'Inputs',
                          'file': 'storm_days.nc'},
                     'precip_z':
                         {'band': None,
                          'nc_name': None,
                          'derivatives': {'requires': ['percent_snow'],
                                          'products': ['precip_vol',
                                                       'rain_z']},
                          'unit_type': 'depth',
                          'description': 'precipitation',
                          'units': 'mm',
                          'calculate': 'mean',
                          'table': 'Results',
                          'file': 'precip.nc'},
                     # currently we have 'precip' that goes to Inputs table
                     # and 'precip_z' on Results
                     'precip':
                         {'band': None,
                          'nc_name': 'precip',
                          'derivatives': {'requires': [],
                                          'products': []},
                          'unit_type': 'depth',
                          'description': 'precipitation',
                          'units': 'mm',
                          'calculate': 'mean',
                          'table': 'Inputs',
                          'file': 'precip.nc'},

                     'percent_snow':
                         {'band': None,
                          'nc_name': 'percent_snow',
                          'derivatives': {'requires': [],
                                          'products': ['precip_vol',
                                                       'precip_z',
                                                       'rain']},
                          'unit_type': 'percent',
                          'description': 'percent of precipitation that is snow',
                          'units': '%',
                          'calculate': 'mean',
                          'table': 'Inputs',
                          'file': 'percent_snow.nc'},
                     'snow_density':
                         {'band': None,
                          'nc_name': 'snow_density',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'density',
                          'description': 'density of storm snow',
                          'units': 'kg/m^3',
                          'calculate': 'mean',
                          'table': 'Inputs',
                          'file': 'snow_density.nc'},
                     'thermal':
                         {'band': None,
                          'nc_name': 'thermal',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'flux',
                          'description': 'thermal radiation',
                          'units': 'W/m^2',
                          'calculate': 'mean',
                          'table': 'Inputs',
                          'file': 'thermal.nc'},
                     'vapor_pressure':
                         {'band': None,
                          'nc_name': 'vapor_pressure',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'pressure',
                          'description': 'vapor pressure',
                          'units': 'Pa',
                          'calculate': 'mean',
                          'table': 'Inputs',
                          'file': 'vapor_pressure.nc'},
                     'wind_speed':
                         {'band': None,
                          'nc_name': 'wind_speed',
                          'derivatives': {'requires': [], 'products': []},
                          'unit_type': 'speed',
                          'description': 'wind_speed',
                          'units': 'm/s',
                          'calculate': 'mean',
                          'table': 'Inputs',
                          'file': 'wind_speed.nc'},

                     # these are the derivative products
                     'swi_vol':
                         {'band': None,
                          'nc_name': None,
                          'derivatives': {'requires': ['swi_z'],
                                          'products': []},
                          'unit_type': 'volumne',
                          'description': 'surface water input volume',
                          'units': 'm^3',
                          'calculate': 'sum',
                          'table': 'Results',
                          'file': None},
                     'swe_avail':
                         {'band': None,
                          'nc_name': None,
                          'derivatives': {'requires': ['swe_z, coldcont'],
                                          'products': []},
                          'unit_type': 'volumne',
                          'description': 'snow water equivalent available for melt',
                          'units': 'm^3',
                          'calculate': 'sum',
                          'table': 'Results',
                          'file': None},
                     'swe_unavail':
                         {'band': None,
                          'nc_name': None,
                          'derivatives': {'requires': ['swe_z, coldcont'],
                                          'products': []},
                          'unit_type': 'volumne',
                          'description': 'snow water equivalent unavailable for melt',
                          'units': 'm^3',
                          'calculate': 'sum',
                          'table': 'Results',
                          'file': None},
                     'swe_vol':
                         {'band': None,
                          'nc_name': None,
                          'derivatives': {'requires': ['swe_z'],
                                          'products': []},
                          'unit_type': 'volumne',
                          'description': 'snow water equivalent volume',
                          'units': 'm^3',
                          'calculate': 'sum',
                          'table': 'Results',
                          'file': None},
                     'precip_vol':
                         {'band': None,
                          'nc_name': None,
                          'derivatives': {'requires': ['precip_z'],
                                          'products': []},
                          'unit_type': 'volume',
                          'description': 'precipitation volume',
                          'units': 'm^3',
                          'calculate': 'sum',
                          'table': 'Results',
                          'file': None},
                     'rain_z':
                         {'band': None,
                          'nc_name': None,
                          'derivatives': {'requires': ['precip_z',
                                                       'percent_snow'],
                                          'products': []},
                          'unit_type': 'depth',
                          'description': 'rain depth',
                          'units': 'mm',
                          'calculate': 'mean',
                          'table': 'Results',
                          'file': None},
                     'snow_line':
                         {'band': None,
                          'nc_name': None,
                          'derivatives': {'requires': ['swe_z'],
                                          'products': []},
                          'unit_type': 'elevation',
                          'description': 'snow line',
                          'units': 'm',
                          'calculate': 'mean',
                          'table': 'Results',
                          'file': None}
                     }

        # snow.nc and em.nc variables
        self.awsm_variables = []

        # smrf variables
        self.smrf_variables = []

        # all named snowav variables
        self.snowav_named_variables = []

        # nc variables
        self.nc_variables = []

        for v in self.vars.keys():
            self.snowav_named_variables.append(v)
            if self.vars[v]['band'] is not None:
                self.awsm_variables.append(v)

            if (self.vars[v]['file'] is not None and
                    self.vars[v]['file'] not in ['em.nc', 'snow.nc']):
                self.smrf_variables.append(v)

            if self.vars[v]['nc_name'] is not None:
                self.nc_variables.append(self.vars[v]['nc_name'])

        # Results table values
        self.snowav_results_variables = []
        for v in self.vars.keys():
            if self.vars[v]['table'] == 'Results':
                self.snowav_results_variables.append(v)
                if self.vars[v]['derivatives']['products'] != []:
                    for d in self.vars[v]['derivatives']['products']:
                        self.snowav_results_variables.append(d)

        # Inputs table values
        self.snowav_inputs_variables = []
        for v in self.vars.keys():
            if self.vars[v]['table'] == 'Inputs':
                self.snowav_inputs_variables.append(v)
                if self.vars[v]['derivatives']['products'] != []:
                    for d in self.vars[v]['derivatives']['products']:
                        self.snowav_inputs_variables.append(d)

        self.cumulative_sum_variables = ['swi_vol', 'precip_vol', 'precip_z',
                                         'swi_z']

        self.process_depth_units = ['coldcont', 'density', 'depth', 'evap_z',
                                    'L_v_E', 'lwc', 'temp_surface',
                                    'temp_lower', 'temp_bulk',
                                    'depth_lower_layer', 'h20_sat', 'R_n', 'H',
                                    'L_v_E', 'G', 'M', 'delta_Q']

    def make_variables(self, properties, edges, columns):
        """ Make a subset of vars with only the requested properties.

        This uses an OrderedDict, and expects properties to be in the
        correct processing order.

        Args
        ------
            properties: list, composed of keys from self.vars
            edges: list, elevation bins
            columns: list, basin names
        """

        self.variables = OrderedDict()
        self.snowav_inputs_variables = []
        self.snowav_results_variables = []

        # make a variable for each desired from the master list, and add
        # an empty dataframe for results
        for p in properties:

            # make lists for each table
            # inputs are processed first, but precip gets placed on Results
            # table
            if self.vars[p]['table'] == 'Inputs':
                self.snowav_inputs_variables.append(p)
                for d in self.vars[p]['derivatives']['products']:
                    self.snowav_inputs_variables.append(d)

            if self.vars[p]['table'] == 'Results':
                self.snowav_results_variables.append(p)
                for d in self.vars[p]['derivatives']['products']:
                    self.snowav_results_variables.append(d)

            # catch invalid properties
            if p not in list(self.vars.keys()):
                raise ValueError('{} not in available snowav '
                                 'properties'.format(p))

            self.variables[p] = deepcopy(self.vars[p])

            if p == 'snow_line':
                index = ['total']
            else:
                index = edges

            self.variables[p]['df'] = pd.DataFrame(np.nan,
                                                   index=index,
                                                   columns=columns)

            # add derivates as well
            if self.vars[p]['derivatives']['products'] != []:
                for d in self.vars[p]['derivatives']['products']:
                    self.variables[d] = deepcopy(self.vars[d])
                    self.variables[d]['df'] = pd.DataFrame(np.nan,
                                                           index=edges,
                                                           columns=columns)



from codecs import open
from os import path
import os
from setuptools import setup, find_packages
from subprocess import check_output

gitVersion = ''
gitPath = ''

# get git describe if in git repository
print('Fetching most recent git tags')
if os.path.exists('./.git'):
    try:
        # if we are in a git repo, fetch most recent tags
        check_output(["git fetch --tags"], shell=True)

    except Exception as e:
        print(e)
        print('Unable to fetch most recent tags')

    try:
        ls_proc = check_output(["git describe --tags"], shell=True, universal_newlines=True)
        gitVersion = ls_proc
        print('Checking most recent version')

    except Exception as e:
        print('Unable to get git tag and hash')

# if not in git repo
else:
    print('Not in git repository')
    gitVersion = ''

# get current working directory to define git path
gitPath = os.getcwd()

# git untracked file to store version and path
fname = os.path.abspath(os.path.expanduser('./snowav/utils/gitinfo.py'))

with open(fname, 'w') as f:
    nchars = len(gitVersion) - 1
    f.write("__gitPath__='{0}'\n".format(gitPath))
    f.write("__gitVersion__='{0}'\n".format(gitVersion[:nchars]))
    f.close()

with open('README.md') as readme_file:
    readme = readme_file.read()

with open('requirements.txt') as f:
    required = f.read().splitlines()

# requirements = []

setup_requirements = []

test_requirements = []

setup(
    name='snowav',
    version='0.11.13',
    description="Snow and Water Model Analysis and Visualization ",
    author="Mark Robertson",
    author_email='mark.robertson@usda.gov',
    url='https://github.com/USDA-ARS-NWRC/snowav',
    packages=['snowav',
              'snowav.config',
              'snowav.framework',
              'snowav.database',
              'snowav.plotting',
              'snowav.inflow',
              'snowav.report',
              'snowav.utils'
              ],

    include_package_data=True,
    package_data={'snowav': ['./config/CoreConfig.ini', './config/recipes.ini']},
    scripts=['./snowav/cli.py'],
    install_requires=required,
    license="CC0 1.0",
    zip_safe=False,
    keywords='snowav',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    setup_requires=setup_requirements,

    entry_points={'console_scripts': ['snowav = cli:run']},
    test_suite='tests'

)




from copy import deepcopy
import os
import numpy as np
from datetime import datetime, timedelta
from snowav.utils.wyhr import calculate_wyhr_from_date
from snowav.utils.OutputReader import iSnobalReader
import netCDF4 as nc

def outputs(run_dirs, wy, properties, start_date = None,
            end_date = None, flight_dates = None, loglevel = None):
    '''
    This uses start_date and end_date to load the snow.nc and em.nc of interest
    within a report period to the outputs format that will be used in process().
    Also returns a clipped run_dirs that only contains paths with the specified
    date range. If start_date and end_date are not supplied, no run_dirs will
    be clipped.

    Note: this is assuming awsm_daily output folders and dates in the snow.nc
    file.

    Args
    -----
    run_dirs : list
        list of run directories
    start_date : datetime
        report period start date (optional)
    end_date : datetime
        report period end date (optional)
    wy : int
        water year
    flight_dates : array
        (optional)

    Returns
    ------
    results : dict

    '''

    dirs = deepcopy(run_dirs)
    snowbands = []
    embands = []
    log = []
    rdict = {}
    outputs = {'dates': [], 'time': []}

    bands_map = {'snow':{'depth': 0,
                         'density': 1,
                         'swe_z': 2,
                         'lwc': 3,
                         'temp_surface': 4,
                         'temp_lower': 5,
                         'temp_bulk': 6,
                         'depth_lower_layer': 7,
                         'h20_sat': 8},
                 'em':{'R_n': 0,
                       'H': 1,
                       'L_v_E': 2,
                       'G': 3,
                       'M': 4,
                       'delta_Q': 5,
                       'evap_z': 6,
                       'melt': 7,
                       'swi_z': 8,
                       'coldcont': 9}}

    for band in properties:
        if band in [*bands_map['snow'].keys()]:
            snowbands.append(bands_map['snow'][band])
        if band in [*bands_map['em'].keys()]:
            embands.append(bands_map['em'][band])
        outputs[band] = []

    start = deepcopy(start_date)
    end = deepcopy(end_date)

    # Run this with standard processing, and forecast processing
    if flight_dates is None:

        for path in dirs:
            snowfile = os.path.join(path, 'snow.nc')

            if loglevel == 'DEBUG':
                log.append(' Reading date: {}'.format(snowfile))

            # Consider making this a warning, with an else: .remove(path)
            # to catch other files that are in these directories
            if not os.path.isfile(snowfile):
                log.append(' {} not a valid file'.format(snowfile))
                print(' {} not a valid file, snowav may '
                    'error...'.format(snowfile))
                run_dirs.remove(path)
                results = {'outputs': outputs,
                           'dirs': dirs,
                           'run_dirs': run_dirs,
                           'rdict': rdict,
                           'log': log}
                return results

            ncf = nc.Dataset(snowfile)

            # Catch 'empty' snow.nc and em.nc file from certain awsm crash
            # scenarios in awsm<=0.10.0
            if 'specific_mass' not in ncf.variables:
                log.append(' No "specific_mass" variable in {}, this may be the result '
                    'of awsm crashing without writing variables to file, '
                    'consider deleting and re-running awsm'.format(snowfile))
                raise Exception(' No "specific_mass" variable in {}'.format(snowfile))

            ta = nc.num2date(ncf.variables['time'][:],ncf.variables['time'].units)
            ncf.close()

            ta = np.sort(ta)

            if start_date is None:
                start = deepcopy(ta[0])

            if end_date is None:
                end = deepcopy(ta[-1])

            for idx,t in enumerate(ta):

                # Only load the rundirs that we need
                if (t.date() >= start.date()) and (t.date() <= end.date()):

                    log.append(' Loading: {}'.format(snowfile))

                    st_hr = calculate_wyhr_from_date(start)
                    en_hr = calculate_wyhr_from_date(end)

                    output = iSnobalReader(path, snowbands = snowbands,
                                           embands = embands, wy = wy,
                                           time_start = st_hr, time_end = en_hr)

                    # Make a dict for wyhr-rundir lookup
                    for ot in output.time:
                        rdict[int(ot)] = path

                    # Apply snow bands to outputs
                    for band in properties:
                        if band in [*bands_map['snow'].keys()]:
                            s = bands_map['snow'][band]
                            outputs[band].append(output.snow_data[s][idx,:,:])
                        if band in [*bands_map['em'].keys()]:
                            e = bands_map['em'][band]
                            outputs[band].append(output.em_data[e][idx,:,:])

                    outputs['dates'].append(output.dates[idx])
                    outputs['time'].append(output.time[idx])

                else:
                    run_dirs.remove(path)

    # Run this when flight updates are present to make custom outputs
    else:

        for path in dirs:
            snowfile = os.path.join(path, 'snow.nc')

            # If the run_dirs isn't empty use it, otherwise remove
            if not os.path.isfile(snowfile):
                raise Exception('{} not a valid file'.format(snowfile))

            ncf = nc.Dataset(snowfile)
            ta = nc.num2date(ncf.variables['time'][:],ncf.variables['time'].units)
            ncf.close()

            for idx,t in enumerate(ta):
                if (t.date() in [x.date() for x in flight_dates]):

                    output = iSnobalReader(path, snowbands=[0,1,2], wy=wy)

                    for ot in output.time:
                        rdict[int(ot)] = path

                    outputs['swe_z'].append(output.snow_data[2][idx,:,:])
                    outputs['depth'].append(output.snow_data[0][idx,:,:])
                    outputs['density'].append(output.snow_data[1][idx,:,:])
                    outputs['dates'].append(output.dates[idx])
                    outputs['time'].append(output.time[idx])

    results = {'outputs': outputs,
               'dirs': dirs,
               'run_dirs': run_dirs,
               'rdict': rdict,
               'log': log}

    return results




import numpy as np
from matplotlib import pyplot as plt
import matplotlib.colors as mcolors
from mpl_toolkits.axes_grid1 import make_axes_locatable
import seaborn as sns
import cmocean
import matplotlib.patches as mpatches
import pandas as pd
import snowav.framework.figures

def cold_content(args, logger):
    '''
    SWE unavailable for melt based on snowpack cold content.

    Args
    ----------
    args : dict
        dictionary with required inputs, see swi() figure for more information.

    logger : list
        snowav logger

    '''
    # print to screen for each figure if desired
    if args['print']:
        print('cold_content() figure args:\n','omitting masks, image, and swe...\n')
        for name in args.keys():
            if name not in ['masks','image','swe']:
                print(name, ': ', args[name])

    masks = args['masks']
    swe = args['swe']
    image = args['image']
    df = args['df']
    plotorder = args['plotorder']
    lims = args['lims']
    edges = args['edges']
    labels = args['labels']
    barcolors = args['barcolors']

    clims2 = (-5,0)
    pmask = masks[plotorder[0]]['mask']
    ixo = pmask == 0
    ixz = swe == 0
    image[ixz] = 1

    mymap1 = plt.cm.Spectral_r
    image[ixo] = np.nan
    mymap1.set_bad('white')
    mymap1.set_over('lightgrey',1)

    sns.set_style('darkgrid')
    sns.set_context("notebook")

    plt.close(1)
    fig,(ax,ax1) = plt.subplots(num=1, figsize=args['figsize'],
                                facecolor = 'white', dpi=args['dpi'],
                                nrows = 1, ncols = 2)

    h = ax.imshow(image, clim=clims2, cmap = mymap1)

    for name in masks:
        ax.contour(masks[name]['mask'],cmap = "Greys",linewidths = 1)

    # Do pretty stuff for the left plot
    h.axes.get_xaxis().set_ticks([])
    h.axes.get_yaxis().set_ticks([])
    h.axes.set_title(args['title'])
    divider = make_axes_locatable(ax)
    cax2 = divider.append_axes("right", size="5%", pad=0.2)
    cbar = plt.colorbar(h, cax = cax2)
    cbar.set_label('[MJ/$m^3$]')
    cbar.ax.tick_params()

    patches = [mpatches.Patch(color='grey', label='snow free')]

    # If there is meaningful snow-free area, include path and label
    if sum(sum(ixz)) > 1000:
        patches = [mpatches.Patch(color='grey', label='snow free')]
        ax.legend(handles=patches, bbox_to_anchor=(lims.pbbx, 0.05),
                  loc=2, borderaxespad=0. )

    # basin total and legend
    ax1.legend(loc=(lims.legx,lims.legy),markerscale = 0.5)

    for iters,name in enumerate(lims.sumorder):

        if args['dplcs'] == 0:
            ukaf = str(np.int(np.nansum(df[name])))
        else:
            ukaf = str(np.round(np.nansum(df[name]),args['dplcs']))

        if iters == 0:
            ax1.bar(range(0,len(edges)),df[name],
                    color = barcolors[iters],
                    edgecolor = 'k',
                    label = labels[name] + ': {} {}'.format(ukaf,args['vollbl']))

        else:
            ax1.bar(range(0,len(edges)),df[name],
                    bottom = pd.DataFrame(df[lims.sumorder[0:iters]]).sum(axis = 1).values,
                    color = barcolors[iters], edgecolor = 'k',
                    label = labels[name] + ': {} {}'.format(ukaf,args['vollbl']))

    if 'ylims' in args.keys():
        ax1.set_ylim(args['ylims'])

    else:
        ylims = ax1.get_ylim()
        max = ylims[1] + ylims[1]*0.5
        min = 0
        ax1.set_ylim((min, max))

    ax1.xaxis.set_ticks(range(0,len(edges)))
    plt.tight_layout()
    ax1.set_xlim((args['xlims'][0]-0.5,args['xlims'][1]-0.5))

    edges_lbl = []
    for i in range(0,len(edges)):
        edges_lbl.append(str(int(edges[int(i)])))

    ax1.set_xticklabels(str(i) for i in edges_lbl)
    for tick in ax1.get_xticklabels():
        tick.set_rotation(30)

    ax1.set_xlim((args['xlims'][0]-0.5,args['xlims'][1]+0.5))

    ax1.set_xlabel('elevation [{}]'.format(args['elevlbl']))
    ax1.set_ylabel('{} '.format(args['vollbl']))
    ax1.yaxis.set_label_position("right")
    ax1.yaxis.tick_right()
    ax1.legend(loc = 2, fontsize = 10)

    for tick in ax1.get_xticklabels():
        tick.set_rotation(30)

    ax1.set_xlim((args['xlims'][0]-0.5,args['xlims'][1]-0.5))
    fig.tight_layout()
    fig.subplots_adjust(top=0.92,wspace = 0.2)

    fig_name_short = 'cold_content_'
    fig_name = '{}{}{}.png'.format(args['figs_path'],fig_name_short,args['directory'])
    if logger is not None:
        logger.info(' saving {}'.format(fig_name))
    snowav.framework.figures.save_fig(fig, fig_name)

    return fig_name_short



from snowav.utils.MidpointNormalize import MidpointNormalize
import numpy as np
import os
from matplotlib import pyplot as plt
import matplotlib.colors as mcolors
from mpl_toolkits.axes_grid1 import make_axes_locatable
import seaborn as sns
import copy
import cmocean
import pandas as pd

import snowav.framework.figures


def image_change(args, logger):
    """
    Makes change in SWE figure, with spatial depth on the left and
    bar volume on right.

    Note: command line tool calls this function. If changes are made
    here update that calls as necessary.

    Args
    ----------
    args : dict
        dictionary with required inputs, see swi() figure for more information.

    logger : dict
        snowav logger

    """

    # print to screen for each figure if desired
    if args['print']:
        print('image_change() figure args:\n', 'omitting masks and image...\n')
        for name in args.keys():
            if name not in ['masks', 'image']:
                print(name, ': ', args[name])

    # These get used more and we'll just assign here
    masks = args['masks']
    delta_swe = args['image']
    df = args['df']
    plotorder = args['plotorder']
    lims = args['lims']
    edges = args['edges']
    labels = args['labels']
    barcolors = args['barcolors']

    vMin, vMax = np.nanpercentile(delta_swe, [args['percent_min'], args['percent_max']])
    colorsbad = plt.cm.Set1_r(np.linspace(0., 1, 1))
    colors1 = cmocean.cm.matter_r(np.linspace(0., 1, 127))
    colors2 = plt.cm.Blues(np.linspace(0, 1, 128))
    colors = np.vstack((colorsbad, colors1, colors2))
    mymap = mcolors.LinearSegmentedColormap.from_list('my_colormap', colors)

    ixf = delta_swe == 0
    delta_swe[ixf] = -100000
    pmask = masks[plotorder[0]]['mask']
    ixo = pmask == 0
    delta_swe[ixo] = np.nan
    cmap = copy.copy(mymap)
    cmap.set_bad('white', 1.)

    sns.set_style('darkgrid')
    sns.set_context("notebook")

    plt.close(6)
    fig, (ax, ax1) = plt.subplots(num=6, figsize=args['figsize'],
                                  dpi=args['dpi'], nrows=1, ncols=2)
    h = ax.imshow(delta_swe, interpolation='none',
                  cmap=cmap, norm=MidpointNormalize(midpoint=0,
                                                    vmin=vMin - 0.01, vmax=vMax + 0.01))

    # Basin boundaries
    for name in masks:
        ax.contour(masks[name]['mask'], cmap="Greys", linewidths=1)

    h.axes.get_xaxis().set_ticks([])
    h.axes.get_yaxis().set_ticks([])
    divider = make_axes_locatable(ax)
    cax = divider.append_axes("right", size="5%", pad=0.2)
    cbar = plt.colorbar(h, cax=cax)
    cbar.set_label(r'$\Delta$ SWE [{}]'.format(args['depthlbl']))

    h.axes.set_title(args['title'])

    if args['dplcs'] == 0:
        tlbl = '{}: {} {}'.format(labels[plotorder[0]],
                                  str(int(df[plotorder[0]].sum())),
                                  args['vollbl'])
    else:
        tlbl = '{}: {} {}'.format(labels[plotorder[0]],
                                  str(np.round(df[plotorder[0]].sum(),
                                               args['dplcs'])), args['vollbl'])

    for iters, name in enumerate(lims.sumorder):

        if args['dplcs'] == 0:
            lbl = '{}: {} {}'.format(labels[name],
                                     str(int(df[name].sum())),
                                     args['vollbl'])
        else:
            lbl = '{}: {} {}'.format(labels[name],
                                     str(np.round(df[name].sum(),
                                                  args['dplcs'])), args['vollbl'])

        if iters == 0:
            ax1.bar(range(0, len(edges)), df[name],
                    color=barcolors[iters],
                    edgecolor='k', label=lbl)

        else:
            ax1.bar(range(0, len(edges)), df[name],
                    bottom=pd.DataFrame(df[lims.sumorder[0:iters]]).sum(axis=1).values,
                    color=barcolors[iters], edgecolor='k', label=lbl)

    ax1.xaxis.set_ticks(range(0, len(edges)))
    plt.tight_layout()
    ax1.set_xlim((args['xlims'][0] - 0.5, args['xlims'][1] - 0.5))

    edges_lbl = []
    for i in range(0, len(edges)):
        edges_lbl.append(str(int(edges[int(i)])))

    ax1.set_xticklabels(str(i) for i in edges_lbl)
    for tick in ax1.get_xticklabels():
        tick.set_rotation(30)

    ax1.set_xlim((args['xlims'][0] - 0.5, args['xlims'][1] - 0.5))

    ylims = ax1.get_ylim()
    max = ylims[1] + abs(ylims[1] - ylims[0])
    min = ylims[0] - abs(ylims[1] - ylims[0]) * 0.1
    ax1.set_ylim((min, max))

    ax1.set_ylabel('{}'.format(args['vollbl']))
    ax1.set_xlabel('elevation [{}]'.format(args['elevlbl']))
    ax1.axes.set_title('Change in Volume')
    ax1.yaxis.set_label_position("right")
    ax1.yaxis.tick_right()

    if len(plotorder) > 1:
        ax1.legend(loc=(lims.legx, lims.legy))

    ax1.text(lims.btx, lims.bty, tlbl, horizontalalignment='center',
             transform=ax1.transAxes, fontsize=10)

    plt.tight_layout()
    fig.subplots_adjust(top=0.88)

    fig_name_short = 'swe_change_'
    fig_name = os.path.join(args['figs_path'],
                            '{}{}.png'.format(fig_name_short,
                                              args['directory']))
    if logger is not None:
        logger.info(' saving {}'.format(fig_name))
    snowav.framework.figures.save_fig(fig, fig_name)

    if 'show' in args.keys() and args['show']:
        plt.show()

    return fig_name_short
